@@ -3,7 +3,7 @@
  *  Brogue
  *
  *  Created by Brian Walker on 6/11/09.
- *  Copyright 2011. All rights reserved.
+ *  Copyright 2012. All rights reserved.
  *  
  *  This file is part of Brogue.
  *
@@ -67,9 +67,9 @@ float strengthModifier(item *theItem) {
 
 float netEnchant(item *theItem) {
 	if (theItem->category & (WEAPON | ARMOR)) {
-		return theItem->enchant1 + strengthModifier(theItem);
+		return ((float) theItem->enchant1) + strengthModifier(theItem);
 	} else {
-		return theItem->enchant1;
+		return ((float) theItem->enchant1);
 	}
 }
 
@@ -80,7 +80,7 @@ short hitProbability(creature *attacker, creature *defender) {
 	short defense = defender->info.defense;
 	short hitProbability;
 	
-	if (defender->status.stuck || (defender->bookkeepingFlags & MONST_CAPTIVE)) {
+	if (defender->status[STATUS_STUCK] || (defender->bookkeepingFlags & MONST_CAPTIVE)) {
 		return 100;
 	}
 	
@@ -106,7 +106,10 @@ short hitProbability(creature *attacker, creature *defender) {
 boolean attackHit(creature *attacker, creature *defender) {
 	
 	// automatically hit if the monster is sleeping or captive or stuck in a web
-	if (defender->status.stuck || defender->bookkeepingFlags & MONST_CAPTIVE) {
+	if (defender->status[STATUS_STUCK]
+		|| defender->status[STATUS_PARALYZED]
+		|| (defender->bookkeepingFlags & MONST_CAPTIVE)) {
+		
 		return true;
 	}
 	
@@ -120,7 +123,7 @@ void monsterShoots(creature *attacker, short targetLoc[2], uchar projChar, color
 	uchar displayChar;
 	color foreColor, backColor, multColor;
 	boolean fastForward = false;
-	char monstName[DCOLS], buf[DCOLS];
+	//char monstName[DCOLS], buf[DCOLS];
 	
 	originLoc[0] = attacker->xLoc;
 	originLoc[1] = attacker->yLoc;
@@ -141,18 +144,18 @@ void monsterShoots(creature *attacker, short targetLoc[2], uchar projChar, color
 		if (pmap[x][y].flags & (HAS_MONSTER | HAS_PLAYER)) {
 			monst = monsterAtLoc(x, y);
 			
-			if (monst && projectileReflects(attacker, monst) && i < DCOLS*2) {
-				if (projectileReflects(attacker, monst)) { // if it scores another reflection roll, reflect at caster
-					numCells = reflectBolt(originLoc[0], originLoc[1], listOfCoordinates, i, true);
-				} else {
-					numCells = reflectBolt(-1, -1, listOfCoordinates, i, false); // otherwise reflect randomly
-				}
-				
-				monsterName(monstName, monst, true);
-				sprintf(buf, "%s deflect%s the arrow", monstName, (monst == &player ? "" : "s"));
-				combatMessage(buf, 0);
-				continue;
-			}
+//			if (monst && projectileReflects(attacker, monst) && i < DCOLS*2) {
+//				if (projectileReflects(attacker, monst)) { // if it scores another reflection roll, reflect at caster
+//					numCells = reflectBolt(originLoc[0], originLoc[1], listOfCoordinates, i, true);
+//				} else {
+//					numCells = reflectBolt(-1, -1, listOfCoordinates, i, false); // otherwise reflect randomly
+//				}
+//				
+//				monsterName(monstName, monst, true);
+//				sprintf(buf, "%s deflect%s the arrow", monstName, (monst == &player ? "" : "s"));
+//				combatMessage(buf, 0);
+//				continue;
+//			}
 			
 			attack(attacker, monst);
 			break;
@@ -215,6 +218,128 @@ void shootWeb(creature *breather, short targetLoc[2], short kindOfWeb) {
 	}
 }
 
+void addMonsterToContiguousMonsterGrid(short x, short y, creature *monst, signed char grid[DCOLS][DROWS]) {
+	short dir, newX, newY;
+	creature *tempMonst;
+	
+	grid[x][y] = true;
+	for (dir=0; dir<4; dir++) {
+		newX = x + nbDirs[dir][0];
+		newY = y + nbDirs[dir][1];
+		
+		if (coordinatesAreInMap(newX, newY) && !grid[newX][newY]) {
+			tempMonst = monsterAtLoc(newX, newY);
+			if (tempMonst && monstersAreTeammates(monst, tempMonst)) {
+				addMonsterToContiguousMonsterGrid(newX, newY, monst, grid);
+			}
+		}
+	}
+}
+
+// Splits a monster in half; this is different from purely cloning it since it halves its HP.
+// The split occurs only if there is a spot adjacent to the contiguous
+// group of monsters that the monster would not avoid.
+// The contiguous group is supplemented with the given (x, y) coordinates, if any;
+// this is so that jellies et al. can spawn behind the player in a hallway.
+void splitMonster(creature *monst, short x, short y) {
+	short i, j, dir, newX, newY, eligibleLocationCount, randIndex;
+	char buf[DCOLS * 3];
+	char monstName[DCOLS];
+	signed char monsterGrid[DCOLS][DROWS], eligibleGrid[DCOLS][DROWS];
+	creature *clone;
+	
+	zeroOutGrid(monsterGrid);
+	zeroOutGrid(eligibleGrid);
+	eligibleLocationCount = 0;
+	
+	// Find the contiguous group of monsters.
+	addMonsterToContiguousMonsterGrid(monst->xLoc, monst->yLoc, monst, monsterGrid);
+	
+	// Add the (x, y) location to the contiguous group, if any.
+	if (x > 0 && y > 0) {
+		monsterGrid[x][y] = true;
+	}
+	
+	// Find the eligible edges around the group of monsters.
+	for (i=0; i<DCOLS; i++) {
+		for (j=0; j<DROWS; j++) {
+			if (monsterGrid[i][j]) {
+				for (dir=0; dir<4; dir++) {
+					newX = i + nbDirs[dir][0];
+					newY = j + nbDirs[dir][1];
+					if (coordinatesAreInMap(newX, newY)
+						&& !eligibleGrid[newX][newY]
+						&& !monsterGrid[newX][newY]
+						&& !(pmap[newX][newY].flags & (HAS_PLAYER | HAS_MONSTER))
+						&& !monsterAvoids(monst, newX, newY)) {
+						
+						eligibleGrid[newX][newY] = true;
+						eligibleLocationCount++;
+					}
+				}
+			}
+		}
+	}
+	
+	// Pick a random location on the eligibleGrid and add the clone there.
+	if (eligibleLocationCount) {
+		randIndex = rand_range(1, eligibleLocationCount);
+		for (i=0; i<DCOLS; i++) {
+			for (j=0; j<DROWS; j++) {
+				if (eligibleGrid[i][j] && !--randIndex) {
+					// Found the spot!
+					
+					monsterName(monstName, monst, true);
+					monst->currentHP = (monst->currentHP + 1) / 2;
+					clone = cloneMonster(monst, false, false);
+					clone->xLoc = i;
+					clone->yLoc = j;
+					pmap[i][j].flags |= HAS_MONSTER;
+					fadeInMonster(clone);
+					refreshSideBar(NULL, false);
+					
+					if (canSeeMonster(monst)) {
+						sprintf(buf, "%s splits in two!", monstName);
+						message(buf, false);
+					}
+					
+					return;
+				}
+			}
+		}
+	}
+}
+
+// This function is called whenever one creature acts aggressively against another in a way that causes damage.
+// This can be things like melee attacks, fire/lightning attacks, throwing a weapon.
+void moralAttack(creature *attacker, creature *defender) {
+	
+	if (defender->currentHP > 0
+		&& !(defender->bookkeepingFlags & MONST_IS_DYING)) {
+		
+		defender->status[STATUS_PARALYZED] = 0;
+		if (defender->status[STATUS_MAGICAL_FEAR]) {
+			defender->status[STATUS_MAGICAL_FEAR] = 1;
+		}
+		defender->status[STATUS_ENTRANCED] = 0;
+		
+		if (attacker == &player
+			&& defender->creatureState == MONSTER_ALLY
+			&& !defender->status[STATUS_DISCORDANT]
+			&& !attacker->status[STATUS_CONFUSED]) {
+		
+			unAlly(defender);
+		}
+		if (defender->info.abilityFlags & MA_CLONE_SELF_ON_DEFEND) {
+			if (distanceBetween(defender->xLoc, defender->yLoc, attacker->xLoc, attacker->yLoc) <= 1) {
+				splitMonster(defender, attacker->xLoc, attacker->yLoc);
+			} else {
+				splitMonster(defender, 0, 0);
+			}
+		}
+	}
+}
+
 boolean playerImmuneToMonster(creature *monst) {
 	if (monst != &player
 		&& rogue.armor
@@ -255,14 +380,14 @@ void specialHit(creature *attacker, creature *defender, short damage) {
 			messageWithColor(buf, &itemMessageColor, false);
 		}
 		if (attacker->info.abilityFlags & MA_HIT_HALLUCINATE) {
-			if (!player.status.hallucinating) {
+			if (!player.status[STATUS_HALLUCINATING]) {
 				combatMessage("you begin to hallucinate", 0);
 			}
-			if (!player.status.hallucinating) {
-				player.maxStatus.hallucinating = 0;
+			if (!player.status[STATUS_HALLUCINATING]) {
+				player.maxStatus[STATUS_HALLUCINATING] = 0;
 			}
-			player.status.hallucinating += 20;
-			player.maxStatus.hallucinating = max(player.maxStatus.hallucinating, player.status.hallucinating);
+			player.status[STATUS_HALLUCINATING] += 20;
+			player.maxStatus[STATUS_HALLUCINATING] = max(player.maxStatus[STATUS_HALLUCINATING], player.status[STATUS_HALLUCINATING]);
 		}
 		
 		if (attacker->info.abilityFlags & MA_HIT_STEAL_FLEE
@@ -271,11 +396,11 @@ void specialHit(creature *attacker, creature *defender, short damage) {
 			&& attacker->currentHP > 0
 			&& attackHit(attacker, defender)) {
 			
-			itemCandidates = numberOfMatchingPackItems(ALL_ITEMS & ~AMULET, 0, (ITEM_EQUIPPED | ITEM_CURSED), false);
+			itemCandidates = numberOfMatchingPackItems(ALL_ITEMS & ~AMULET, 0, (ITEM_EQUIPPED), false);
 			if (itemCandidates) {
 				randItemIndex = rand_range(1, itemCandidates);
 				for (theItem = packItems->nextItem; theItem != NULL; theItem = theItem->nextItem) {
-					if (!(theItem->flags & (ITEM_CURSED | ITEM_EQUIPPED)) && !(theItem->category & AMULET)) {
+					if (!(theItem->flags & (ITEM_EQUIPPED)) && !(theItem->category & AMULET)) {
 						if (randItemIndex == 1) {
 							break;
 						} else {
@@ -307,14 +432,14 @@ void specialHit(creature *attacker, creature *defender, short damage) {
 		}
 	}
 	if (attacker->info.abilityFlags & MA_POISONS && damage > 0) {
-		if (defender == &player && !player.status.poisoned) {
+		if (defender == &player && !player.status[STATUS_POISONED]) {
 			combatMessage("scalding poison fills your veins", &badMessageColor);
 		}
-		if (!defender->status.poisoned) {
-			defender->maxStatus.poisoned = 0;
+		if (!defender->status[STATUS_POISONED]) {
+			defender->maxStatus[STATUS_POISONED] = 0;
 		}
-		defender->status.poisoned = max(defender->status.poisoned, damage);
-		defender->maxStatus.poisoned = max(defender->maxStatus.poisoned, defender->status.poisoned);
+		defender->status[STATUS_POISONED] = max(defender->status[STATUS_POISONED], damage);
+		defender->maxStatus[STATUS_POISONED] = defender->info.maxHP;
 	}
 }
 
@@ -340,7 +465,7 @@ short runicWeaponChance(item *theItem, boolean customEnchantLevel, float enchant
 		return 15;
 	}
 	if (!customEnchantLevel) {
-		enchantLevel = netEnchant(theItem);
+		enchantLevel = (float) netEnchant(theItem);
 	}
 	
 	rootChance = effectChances[runicType];
@@ -348,9 +473,9 @@ short runicWeaponChance(item *theItem, boolean customEnchantLevel, float enchant
 	// Innately high-damage weapon types are less likely to trigger runic effects.
 	adjustedBaseDamage = (theItem->damage.lowerBound + theItem->damage.upperBound) / 2;
 	if (theItem->flags & ITEM_ATTACKS_SLOWLY) {
-		adjustedBaseDamage /= 2; // normalize as though they attacked once per turn instead of every other turn
+		adjustedBaseDamage /= 2; // Normalize as though they attacked once per turn instead of every other turn.
 	}
-	modifier = 1.0 - adjustedBaseDamage / 18.0;
+	modifier = 1.0 - min(0.99, ((float) adjustedBaseDamage) / 18.0);
 	rootChance *= modifier;
 	
 	chance = 100 - (short) (100 * pow(1.0 - rootChance, enchantLevel)); // good runic
@@ -360,7 +485,9 @@ short runicWeaponChance(item *theItem, boolean customEnchantLevel, float enchant
 		chance = 100 - (100 - chance) * (100 - chance) / 100;
 	}
 	
-	chance = clamp(chance, 1, 100);
+	// The lowest percent change that a weapon will ever have is its enchantment level (if greater than 0).
+	// That is so that even really heavy weapons will improve at least 1% per enchantment.
+	chance = clamp(chance, max(1, (short) enchantLevel), 100);
 	
 	return chance;
 }
@@ -377,7 +504,7 @@ void magicWeaponHit(creature *defender, item *theItem, boolean backstabbed) {
 	creature *newMonst;
 	
 	// If the defender is already dead, proceed only if the runic is speed or multiplicity.
-	// (Everything else acts on the victim, which would be overkill.)
+	// (Everything else acts on the victim, which would literally be overkill.)
 	if ((defender->bookkeepingFlags & MONST_IS_DYING)
 		&& theItem->enchant2 != W_SPEED
 		&& theItem->enchant2 != W_MULTIPLICITY) {
@@ -418,14 +545,14 @@ void magicWeaponHit(creature *defender, item *theItem, boolean backstabbed) {
 				break;
 			case W_SLAYING:
 			case W_QUIETUS:
-				inflictDamage(&player, defender, defender->currentHP, 0);
+				inflictLethalDamage(&player, defender);
 				sprintf(buf, "%s suddenly dies", monstName);
 				buf[DCOLS] = '\0';
 				combatMessage(buf, messageColorFromVictim(defender));
 				break;
 			case W_PARALYSIS:
-				defender->status.paralyzed = max(defender->status.paralyzed, weaponParalysisDuration(enchant));
-				defender->maxStatus.paralyzed = defender->status.paralyzed;
+				defender->status[STATUS_PARALYZED] = max(defender->status[STATUS_PARALYZED], weaponParalysisDuration(enchant));
+				defender->maxStatus[STATUS_PARALYZED] = defender->status[STATUS_PARALYZED];
 				if (canSeeMonster(defender)) {
 					sprintf(buf, "%s is frozen in place", monstName);
 					buf[DCOLS] = '\0';
@@ -457,7 +584,7 @@ void magicWeaponHit(creature *defender, item *theItem, boolean backstabbed) {
 					newMonst->ticksUntilTurn = 100;
 					newMonst->info.accuracy = player.info.accuracy + 5 * netEnchant(theItem);
 					newMonst->info.damage = player.info.damage;
-					newMonst->status.lifespanRemaining = newMonst->maxStatus.lifespanRemaining = weaponImageDuration(enchant);
+					newMonst->status[STATUS_LIFESPAN_REMAINING] = newMonst->maxStatus[STATUS_LIFESPAN_REMAINING] = weaponImageDuration(enchant);
 					if (strLenWithoutEscapes(theItemName) <= 6) {
 						sprintf(newMonst->info.monsterName, "spectral %s", theItemName);
 					} else {
@@ -493,8 +620,8 @@ void magicWeaponHit(creature *defender, item *theItem, boolean backstabbed) {
 				}
 				break;
 			case W_CONFUSION:
-				defender->status.confused = max(defender->status.confused, weaponConfusionDuration(enchant));
-				defender->maxStatus.confused = defender->status.confused;
+				defender->status[STATUS_CONFUSED] = max(defender->status[STATUS_CONFUSED], weaponConfusionDuration(enchant));
+				defender->maxStatus[STATUS_CONFUSED] = defender->status[STATUS_CONFUSED];
 				if (canSeeMonster(defender)) {
 					sprintf(buf, "%s looks very confused", monstName);
 					buf[DCOLS] = '\0';
@@ -505,7 +632,7 @@ void magicWeaponHit(creature *defender, item *theItem, boolean backstabbed) {
 				heal(defender, 50);
 				break;
 			case W_PLENTY:
-				newMonst = cloneMonster(defender, true);
+				newMonst = cloneMonster(defender, true, true);
 				if (newMonst) {
 					flashMonster(newMonst, effectColors[enchantType], 100);
 				}
@@ -519,7 +646,7 @@ void magicWeaponHit(creature *defender, item *theItem, boolean backstabbed) {
 void attackVerb(char returnString[DCOLS], creature *attacker, short hitPercentile) {
 	short verbCount, increment;
 	
-	if (attacker != &player && (player.status.hallucinating || !canSeeMonster(attacker))) {
+	if (attacker != &player && (player.status[STATUS_HALLUCINATING] || !canSeeMonster(attacker))) {
 		strcpy(returnString, "hits");
 		return;
 	}
@@ -556,11 +683,11 @@ void applyArmorRunicEffect(char returnString[DCOLS], creature *attacker, short *
 	switch (rogue.armor->enchant2) {
 		case A_MULTIPLICITY:
 			if (melee && !(attacker->info.flags & MONST_INANIMATE) && rand_percent(33)) {
-				for (i = 0; i < 3; i++) {
-					monst = cloneMonster(attacker, false);
+				for (i = 0; i < armorImageCount(enchant); i++) {
+					monst = cloneMonster(attacker, false, true);
 					monst->bookkeepingFlags |= (MONST_FOLLOWER | MONST_BOUND_TO_LEADER | MONST_DOES_NOT_TRACK_LEADER);
 					monst->info.flags |= MONST_DIES_IF_NEGATED;
-					monst->bookkeepingFlags &= ~MONST_JUST_SUMMONED;
+					monst->bookkeepingFlags &= ~(MONST_JUST_SUMMONED | MONST_SEIZED | MONST_SEIZING);
 					monst->info.abilityFlags &= ~MA_CAST_SUMMON; // No summoning by spectral images. Gotta draw the line!
 					monst->leader = &player;
 					monst->creatureState = MONSTER_ALLY;
@@ -573,7 +700,7 @@ void applyArmorRunicEffect(char returnString[DCOLS], creature *attacker, short *
 					monst->info.foreColor = &spectralImageColor;
 					
 					// Temporary guest!
-					monst->status.lifespanRemaining = monst->maxStatus.lifespanRemaining = 3;
+					monst->status[STATUS_LIFESPAN_REMAINING] = monst->maxStatus[STATUS_LIFESPAN_REMAINING] = 3;
 					monst->currentHP = monst->info.maxHP = 1;
 					monst->info.defense = 0;
 					
@@ -692,8 +819,8 @@ void applyArmorRunicEffect(char returnString[DCOLS], creature *attacker, short *
 // returns whether the attack hit
 boolean attack(creature *attacker, creature *defender) {
 	short damage, transferenceAmount, poisonDamage;
-	char buf[COLS], buf2[COLS], attackerName[COLS], defenderName[COLS], verb[DCOLS], explicationClause[DCOLS] = "", armorRunicString[DCOLS];
-	boolean sneakAttack, defenderWasAsleep, degradesAttackerWeapon, sightUnseen;
+	char buf[COLS*2], buf2[COLS*2], attackerName[COLS], defenderName[COLS], verb[DCOLS], explicationClause[DCOLS] = "", armorRunicString[DCOLS*3];
+	boolean sneakAttack, defenderWasAsleep, defenderWasParalyzed, degradesAttackerWeapon, sightUnseen;
 	
 	if (attacker->info.abilityFlags & MA_KAMIKAZE) {
 		addExperience(attacker->info.expForKilling);
@@ -709,7 +836,7 @@ boolean attack(creature *attacker, creature *defender) {
 	
 	sightUnseen = !(canSeeMonster(attacker) || canSeeMonster(defender));
 	
-	if (defender->status.levitating && (attacker->info.flags & MONST_RESTRICTED_TO_LIQUID)) {
+	if (defender->status[STATUS_LEVITATING] && (attacker->info.flags & MONST_RESTRICTED_TO_LIQUID)) {
 		return false; // aquatic or other liquid-bound monsters cannot attack flying opponents
 	}
 	
@@ -717,9 +844,9 @@ boolean attack(creature *attacker, creature *defender) {
 		rogue.disturbed = true;
 	}
 	
-	defender->status.entranced = 0;
-	if (defender->status.magicalFear) {
-		defender->status.magicalFear = 1;
+	defender->status[STATUS_ENTRANCED] = 0;
+	if (defender->status[STATUS_MAGICAL_FEAR]) {
+		defender->status[STATUS_MAGICAL_FEAR] = 1;
 	}
 	
 	if (attacker != &player && defender == &player && attacker->creatureState == MONSTER_WANDERING) {
@@ -728,6 +855,7 @@ boolean attack(creature *attacker, creature *defender) {
 	
 	sneakAttack = (defender != &player && attacker == &player && (defender->creatureState == MONSTER_WANDERING) ? true : false);
 	defenderWasAsleep = (defender != &player && (defender->creatureState == MONSTER_SLEEPING) ? true : false);
+	defenderWasParalyzed = defender->status[STATUS_PARALYZED] > 0;
 	
 	monsterName(attackerName, attacker, true);
 	monsterName(defenderName, defender, true);
@@ -742,11 +870,11 @@ boolean attack(creature *attacker, creature *defender) {
 		return false;
 	}
 	
-	if (sneakAttack || defenderWasAsleep || attackHit(attacker, defender)) {
-		
+	if (sneakAttack || defenderWasAsleep || defenderWasParalyzed || attackHit(attacker, defender)) {
+		// If the attack hit:
 		damage = (defender->info.flags & MONST_IMMUNE_TO_WEAPONS ? 0 : randClump(attacker->info.damage));
 		
-		if (sneakAttack || defenderWasAsleep) {
+		if (sneakAttack || defenderWasAsleep || defenderWasParalyzed) {
 			// The defender doesn't hit back this turn because he's still flat-footed.
 			defender->ticksUntilTurn += max(defender->movementSpeed, defender->attackSpeed);
 			defender->creatureState = MONSTER_TRACKING_SCENT; // Wake up!
@@ -788,16 +916,18 @@ boolean attack(creature *attacker, creature *defender) {
 			if (attacker == &player) {
 				rogue.disturbed = true;
 			}
+		} else if (defenderWasParalyzed) {
+			sprintf(explicationClause, " while $HESHE %s paralyzed", (defender == &player ? "are" : "is"));
 		} else if (defenderWasAsleep) {
-			sprintf(explicationClause, " while %s sleep%s", monsterText[defender->info.monsterID].pronoun,
-					(defender == &player ? "" : "s"));
+			strcpy(explicationClause, " in $HISHER sleep");
 		} else if (sneakAttack) {
-			strcpy(explicationClause, " from the shadows");
-		} else if (defender->status.stuck || defender->bookkeepingFlags & MONST_CAPTIVE) {
+			strcpy(explicationClause, ", catching $HIMHER unaware");
+		} else if (defender->status[STATUS_STUCK] || defender->bookkeepingFlags & MONST_CAPTIVE) {
 			sprintf(explicationClause, " while %s dangle%s helplessly",
-					(canSeeMonster(defender) ? monsterText[defender->info.monsterID].pronoun : "it"),
+					(canSeeMonster(defender) ? "$HESHE" : "it"),
 					(defender == &player ? "" : "s"));
 		}
+		resolvePronounEscapes(explicationClause, defender);
 		
 		if ((attacker->info.abilityFlags & MA_POISONS) && damage > 0) {
 			poisonDamage = damage;
@@ -805,7 +935,7 @@ boolean attack(creature *attacker, creature *defender) {
 		}
 		
 		if (inflictDamage(attacker, defender, damage, &red)) { // if the attack killed the defender
-			if (defenderWasAsleep || sneakAttack) {
+			if (defenderWasAsleep || sneakAttack || defenderWasParalyzed) {
 				sprintf(buf, "%s %s %s%s", attackerName,
 						((defender->info.flags & MONST_INANIMATE) ? "destroyed" : "dispatched"),
 						defenderName,
@@ -837,9 +967,7 @@ boolean attack(creature *attacker, creature *defender) {
 				&& !--rogue.weapon->charges) {
 				
 				rogue.weapon->flags |= ITEM_IDENTIFIED;
-				if (!(rogue.weapon->flags & ITEM_RUNIC) || (rogue.weapon->flags & ITEM_RUNIC_IDENTIFIED)) {
-					rogue.weapon->flags &= ~ITEM_CAN_BE_IDENTIFIED;
-				}
+				updateIdentifiableItems();
 				messageWithColor("you are now familiar enough with your weapon to identify it.", &itemMessageColor, false);
 				itemName(rogue.weapon, buf2, true, true, NULL);
 				sprintf(buf, "%s %s.", (rogue.weapon->quantity > 1 ? "they are" : "it is"), buf2);
@@ -871,14 +999,10 @@ boolean attack(creature *attacker, creature *defender) {
 			}
 		}
 		if (attacker == &player && rogue.weapon && (rogue.weapon->flags & ITEM_RUNIC)) { // moved to here
-			magicWeaponHit(defender, rogue.weapon, sneakAttack || defenderWasAsleep);
+			magicWeaponHit(defender, rogue.weapon, sneakAttack || defenderWasAsleep || defenderWasParalyzed);
 		}
 		
-		if (defender->currentHP > 0
-			&& !(defender->bookkeepingFlags & MONST_IS_DYING)
-			&& (defender->info.abilityFlags & MA_CLONE_SELF_ON_DEFEND)) {
-			splitMonster(defender);
-		}
+		moralAttack(attacker, defender);
 		
 		if (degradesAttackerWeapon
 			&& attacker == &player
@@ -913,27 +1037,28 @@ boolean attack(creature *attacker, creature *defender) {
 
 void addExperience(unsigned long exp) {	
 	rogue.experience += exp;
-	if (rogue.experience >= levelPoints[rogue.experienceLevel - 1] && rogue.experienceLevel < MAX_EXP_LEVEL) {
-		while (rogue.experience >= levelPoints[rogue.experienceLevel - 1] && rogue.experienceLevel < MAX_EXP_LEVEL) {
-			rogue.experienceLevel++;
-			player.info.maxHP += 5;
-			player.currentHP += (5 * player.currentHP / (player.info.maxHP - 5));
-			updatePlayerRegenerationDelay();
-			player.info.accuracy += 10;
-		}
+	while (rogue.experience >= levelPoints[rogue.experienceLevel - 1] && rogue.experienceLevel < MAX_EXP_LEVEL) {
+		rogue.experienceLevel++;
+		player.info.maxHP += 5;
+		player.currentHP += (5 * player.currentHP / (player.info.maxHP - 5));
+		updatePlayerRegenerationDelay();
+		player.info.accuracy += 10;
 		rogue.gainedLevel = true;
 	}
 }
 
+// Gets the length of a string without the four-character color escape sequences, since those aren't displayed.
 short strLenWithoutEscapes(char *str) {
 	short i, count;
 	
 	count = 0;
-	for (i=0; str[i]; i++) {
+	for (i=0; str[i];) {
 		if (str[i] == COLOR_ESCAPE) {
-			count -= 4;
+			i += 4;
+			continue;
 		}
 		count++;
+		i++;
 	}
 	return count;
 }
@@ -1068,6 +1193,10 @@ boolean anyoneWantABite(creature *decedent) {
 
 #define MIN_FLASH_STRENGTH	50
 
+void inflictLethalDamage(creature *attacker, creature *defender) {
+	inflictDamage(attacker, defender, defender->currentHP + (10 * defender->status[STATUS_SHIELDED]), NULL);
+}
+
 // returns true if this was a killing stroke; does NOT free the pointer, but DOES remove it from the monster chain
 // flashColor indicates the color that the damage will cause the creature to flash
 boolean inflictDamage(creature *attacker, creature *defender, short damage, const color *flashColor) {
@@ -1078,6 +1207,16 @@ boolean inflictDamage(creature *attacker, creature *defender, short damage, cons
 		return false;
 	}
 	
+	if (defender->status[STATUS_SHIELDED]) {
+		if (defender->status[STATUS_SHIELDED] > damage * 10) {
+			defender->status[STATUS_SHIELDED] -= damage * 10;
+			damage = 0;
+		} else {
+			damage -= (defender->status[STATUS_SHIELDED] + 9) / 10;
+			defender->status[STATUS_SHIELDED] = defender->maxStatus[STATUS_SHIELDED] = 0;
+		}
+	}
+	
 	defender->bookkeepingFlags &= ~MONST_ABSORBING; // Stop eating a corpse if you are getting hurt.
 	
 	// bleed all over the place, proportionately to damage inflicted:
@@ -1093,9 +1232,6 @@ boolean inflictDamage(creature *attacker, creature *defender, short damage, cons
 	if (defender != &player && defender->creatureState == MONSTER_SLEEPING) {
 		wakeUp(defender);
 		chooseNewWanderDestination(defender);
-	} else if (defender != &player && attacker == &player && defender->creatureState == MONSTER_ALLY) {
-		// totally not your friend anymore
-		unAlly(defender);
 	}
 	
 	if (defender == &player && rogue.easyMode) {
@@ -1117,6 +1253,7 @@ boolean inflictDamage(creature *attacker, creature *defender, short damage, cons
 		if (defender != &player && defender->creatureState != MONSTER_ALLY
 			&& defender->info.flags & MONST_FLEES_NEAR_DEATH
 			&& defender->info.maxHP / 4 >= defender->currentHP) {
+			
 			defender->creatureState = MONSTER_FLEEING;
 			chooseNewWanderDestination(defender);
 		}
@@ -1125,7 +1262,7 @@ boolean inflictDamage(creature *attacker, creature *defender, short damage, cons
 		}
 	}
 	
-	refreshSideBar(NULL);
+	refreshSideBar(NULL, false);
 	return killed;
 }
 
@@ -1155,9 +1292,10 @@ void killCreature(creature *decedent, boolean administrativeDeath) {
 	if (!administrativeDeath && (decedent->info.abilityFlags & MA_DF_ON_DEATH)) {
 		spawnDungeonFeature(decedent->xLoc, decedent->yLoc, &dungeonFeatureCatalog[decedent->info.DFType], true, false);
 		
-		if (monsterText[decedent->info.monsterID].DFMessage && canSeeMonster(decedent)) {
+		if (monsterText[decedent->info.monsterID].DFMessage[0] && canSeeMonster(decedent)) {
 			monsterName(monstName, decedent, true);
 			sprintf(buf, "%s %s", monstName, monsterText[decedent->info.monsterID].DFMessage);
+			resolvePronounEscapes(buf, decedent);
 			message(buf, false);
 		}
 	}
@@ -1169,17 +1307,24 @@ void killCreature(creature *decedent, boolean administrativeDeath) {
 			&& decedent->creatureState == MONSTER_ALLY
 			&& !canSeeMonster(decedent)
 			&& !(decedent->info.flags & MONST_INANIMATE)
-			&& !(decedent->bookkeepingFlags & MONST_BOUND_TO_LEADER)) {
+			&& !(decedent->bookkeepingFlags & MONST_BOUND_TO_LEADER)
+			&& !decedent->carriedMonster) {
 			messageWithColor("you feel a sense of loss.", &badMessageColor, false);
 		}
 		x = decedent->xLoc;
 		y = decedent->yLoc;
-		pmap[x][y].flags &= ~HAS_MONSTER;
+		if (decedent->bookkeepingFlags & MONST_IS_DORMANT) {
+			pmap[x][y].flags &= ~HAS_DORMANT_MONSTER;
+		} else {
+			pmap[x][y].flags &= ~HAS_MONSTER;
+		}
+		removeMonsterFromChain(decedent, dormantMonsters);
 		removeMonsterFromChain(decedent, monsters);
+		
 		decedent->nextCreature = graveyard->nextCreature;
 		graveyard->nextCreature = decedent;
 		
-		if (!administrativeDeath) {
+		if (!administrativeDeath && !(decedent->bookkeepingFlags & MONST_IS_DORMANT)) {
 			// Was there another monster inside?
 			if (decedent->carriedMonster) {
 				// Insert it into the chain.
@@ -1210,37 +1355,59 @@ void killCreature(creature *decedent, boolean administrativeDeath) {
 	}
 }
 
-// basically runs a simplified deterministic combat simulation against a hypothetical
+// Basically runs a simplified deterministic melee combat simulation against a hypothetical
 // monster with infinite HP (the dummy) and returns the amount of damage the tested
 // monster deals before succumbing.
-short monsterPower(creature *theMonst) {
+short monsterPower(const creature *theMonst) {
 	short damageDealt[2] = {0, 0};
-	short turnsParalyzed[2] = {theMonst->status.paralyzed, 0};
-	short turnsPoisoned[2] = {theMonst->status.poisoned, 0};
-	short k;
+	short statuses[NUMBER_OF_STATUS_EFFECTS];
+	short ticksTillTurn[2] = {0, 0}, speed;
+	short i, k;
 	double damagePerHit[2];
 	double hitChance[2];
 	
-	damagePerHit[0] = (theMonst->info.damage.lowerBound + theMonst->info.damage.upperBound) / 2 * 100 / (theMonst->info.attackSpeed);
-	damagePerHit[1] = 10;
-	hitChance[0] = theMonst->info.accuracy * pow(DEFENSE_FACTOR, 100); // assumes the dummy has 100 armor
-	hitChance[1] = theMonst->info.accuracy * pow(DEFENSE_FACTOR, theMonst->info.defense);
-	turnsParalyzed[1] += theMonst->status.hasted / 2;
-
+	// [0] is the tested monster and [1] is the dummy.
+	// damageDealt measures how much damage each contestant has inflicted.
 	
-	while (damageDealt[1] < theMonst->currentHP) {
+	for (i=0; i<NUMBER_OF_STATUS_EFFECTS; i++) {
+		statuses[i] = theMonst->status[i];
+	}
+	
+	damagePerHit[0] = (theMonst->info.damage.lowerBound + theMonst->info.damage.upperBound) / 2;
+	damagePerHit[1] = 10;
+	hitChance[0] = theMonst->info.accuracy * pow(DEFENSE_FACTOR, 100); // Assumes the dummy has 100 armor.
+	hitChance[1] = theMonst->info.accuracy * pow(DEFENSE_FACTOR, theMonst->info.defense);
+
+	while (damageDealt[1] < theMonst->currentHP) { // Loop until the dummy kills the monster in the simulation.
 		for (k=0; k<=1; k++) { // k is whose turn it is
-			if (turnsPoisoned[!k]) {
-				damageDealt[k]++;
-				turnsPoisoned[!k]--;
+			
+			if (k==0) { // monster
+				speed = theMonst->info.attackSpeed;
+				
+				if (statuses[STATUS_POISONED]) {
+					damageDealt[1]++; // dummy gets credit for 1 point of damage
+				}
+				if (statuses[STATUS_HASTED]) {
+					speed /= 2;
+				}
+				if (statuses[STATUS_SLOWED]) {
+					speed *= 2;
+				}
+				
+				for (i=0; i<NUMBER_OF_STATUS_EFFECTS; i++) {
+					if (statuses[i] > 0) {
+						statuses[i]--;
+					}
+				}
 			}
 			
-			if (turnsParalyzed[k]) {
-				turnsParalyzed[k]--;
-			} else {
-				damageDealt[k] += damagePerHit[k] * hitChance[k];
+			while (ticksTillTurn[k] <= 0) {
+				if (k == 1 || !statuses[STATUS_PARALYZED]) {
+					damageDealt[k] += damagePerHit[k] * hitChance[k];
+				}
+				ticksTillTurn[k] += k ? 100 : speed;
 			}
-			
+			ticksTillTurn[k] -= 100;
 		}
 	}
 	
