@@ -2,7 +2,7 @@
 
 #ifdef BROGUE_TCOD
 #include "libtcod.h"
-TCOD_renderer_t renderer=TCOD_RENDERER_SDL;
+TCOD_renderer_t renderer=TCOD_RENDERER_SDL; // the sdl renderer is more reliable
 short brogueFontSize = 3;
 #endif
 
@@ -19,6 +19,22 @@ int main(int argc, char *argv[])
 
 	int i;
 	for (i = 1; i < argc; i++) {
+		if (strcmp(argv[i], "--scores") == 0) {
+			// just dump the scores and quit!
+			dumpScores();
+			return 0;
+		}
+
+		if (strcmp(argv[i], "--seed") == 0) {
+			// pick a seed!
+			if (i + 1 < argc) {
+				int seed = atoi(argv[i + 1]);
+				if (seed != 0) {
+					i++;
+				}
+			}
+		}
+
 #ifdef BROGUE_TCOD
 		if (strcmp(argv[i], "--SDL") == 0 || strcmp(argv[i], "-s") == 0) {
 			renderer = TCOD_RENDERER_SDL;
@@ -36,6 +52,7 @@ int main(int argc, char *argv[])
 #endif
 	}
 	
+	loadKeymap();
 	currentConsole.gameLoop();
 	
 	return 0;
